# mypy: ignore-errors
# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# pylint: skip-file
# fmt: off
# isort: skip_file
from .....schema_classes import DataHubMetricCubeDefinitionClass
from .....schema_classes import DataHubMetricCubeDimensionClass
from .....schema_classes import DataHubMetricCubeDimensionsClass
from .....schema_classes import DataHubMetricCubeEventClass
from .....schema_classes import DataHubMetricCubeMeasureClass
from .....schema_classes import DataHubMetricCubeMeasuresClass
from .....schema_classes import DataHubMetricCubeOriginClass
from .....schema_classes import DataHubMetricSourceTypeClass


DataHubMetricCubeDefinition = DataHubMetricCubeDefinitionClass
DataHubMetricCubeDimension = DataHubMetricCubeDimensionClass
DataHubMetricCubeDimensions = DataHubMetricCubeDimensionsClass
DataHubMetricCubeEvent = DataHubMetricCubeEventClass
DataHubMetricCubeMeasure = DataHubMetricCubeMeasureClass
DataHubMetricCubeMeasures = DataHubMetricCubeMeasuresClass
DataHubMetricCubeOrigin = DataHubMetricCubeOriginClass
DataHubMetricSourceType = DataHubMetricSourceTypeClass

# fmt: on
