{
  "type": "record",
  "Aspect": {
    "name": "aiInferenceMetadata"
  },
  "name": "AiInferenceMetadata",
  "namespace": "com.linkedin.pegasus2avro.ai",
  "fields": [
    {
      "type": [
        "null",
        {
          "type": "record",
          "name": "GlossaryTermsInferenceMetadata",
          "namespace": "com.linkedin.pegasus2avro.ai",
          "fields": [
            {
              "Searchable": {
                "fieldName": "glossaryTermsLastInferredAt"
              },
              "type": "long",
              "name": "lastInferredAt",
              "doc": "Timestamp of the last inference in milliseconds."
            },
            {
              "Searchable": {
                "fieldName": "glossaryTermsVersion"
              },
              "type": "long",
              "name": "version",
              "doc": "Monotonically incrementing version number. Once bumped, will re-trigger a new inference."
            }
          ]
        }
      ],
      "name": "glossaryTermsInference",
      "default": null,
      "doc": "Information about a glossary terms inference."
    }
  ],
  "doc": "AI Inference Metadata of various types."
}